"""This module is used for CLI module initialization and execution"""
import os
import inspect
import sys
from multiprocessing import  Pipe
from threading import Thread
import signal
import imp

sys.path.insert(1, imp.find_module('lightbulb')[1]+'/modules')

OBJECTA = None
OBJECTB = None
SHARED1 = None
SHARED2 = None
channel = None
TARGET_A = None
TARGET_B = None

def signal_handler(signal, frame):
    """
    Terminate servers on SIGINT
    Args:
        signal (int): The requested signal.
        frame (func): The signal handler
    Returns:
        None
    """
    if TARGET_A is not None:
        if TARGET_A.isAlive():
            TARGET_A.join(0)
    if TARGET_B is not None:
        if TARGET_B.isAlive():
            TARGET_B.join(0)
    sys.exit(0)



def operate_single(module, configuration):
    """
   Initiates the execution of a module
   Args:
       module (class): The module class to be initiated
       configuration (dict): The module options
   Returns:
       list: A list with the results and stats as tuples
   """
    initmodule = module(configuration)
    initmodule.learn()
    name, bypass = initmodule.getresult()
    return [(name, bypass)] + initmodule.stats()

def operate_multi(module, configuration, channel, cross):
    """
    Initiates the execution of a module in parallel
    Args:
        module (class): The module class to be initiated
        configuration (dict): The module options
        channel (list): A shared memory for intermodule communication
        cross (int): The identifier for this instance
    Returns:
        None
    """
    initmodule = module(configuration, channel, cross)
    if cross == 1:
        channel[4] = initmodule
        initmodule.learn()
        name, bypass = initmodule.getresult()
        channel[6] = [(name, bypass)]+initmodule.stats()
    else:
        channel[5] = initmodule
        initmodule.learn()

def manage(module_name, configuration):
    """
    This function manages the module initialization and execution
    Args:
        module_name (class): The module class to be initiated
        configuration (dict): The module options
    Returns:
       list: A list with the results and stats as tuples
    """
    global OBJECTA, OBJECTB, SHARED1, SHARED2, TARGET_A, TARGET_B
    signal.signal(signal.SIGINT, signal_handler)
    module = __import__(module_name)
    classmembers = [classmodule for classmodule in inspect.getmembers(module, inspect.isclass)
                    if classmodule[0] == 'Module' or classmodule[0] == 'Module_A'
                    or classmodule[0] == 'Module_B']
    if len(classmembers) == 1:
        return  operate_single(classmembers[0][1], configuration)
    else:
        target_a_pipe, target_b_pipe = Pipe()
        result = []
        channel = [target_a_pipe, target_b_pipe, OBJECTA, OBJECTB, SHARED1, SHARED2, result]
        if classmembers[0][0] == "Module_A":
            cross = 1
        else:
            cross = 2
        TARGET_A = Thread(target=operate_multi,
                          args=(classmembers[0][1], configuration, channel, cross))
        TARGET_A.setDaemon(True)
        TARGET_A.start()
        if classmembers[1][0] == "Module_A":
            cross = 1
        else:
            cross = 2
        TARGET_B = Thread(target=operate_multi,
                          args=(classmembers[1][1], configuration, channel, cross))
        TARGET_B.setDaemon(True)
        TARGET_B.start()
        while True:
            TARGET_A.join(600)
            if not TARGET_A.isAlive():
                break
        # End Cross _check
        if TARGET_B.isAlive():
            TARGET_B.join(0)
        return  channel[6]


