from lightbulb.core.base import CORE, createalphabet
from lightbulb.core.utils.browsersocket_hander import BrowserSocketHandler

META = {
    'author': 'George Argyros, Ioannis Stais',
    'description': 'Find differences between two browser JavaScript parsers',
    'type': 'SFADiff',
    'options': [
        ('FILE_A', None, True, 'File containting regular expressions/grammar'),
        ('FILE_TYPE_A', "FLEX", True, 'File type (e.g FLEX, GRAMMAR, FST)'),
        ('WSPORT_A', "8001", True, 'The Web Socket server port'),
        ('WBPORT_A', "8081", True, 'The Web server port'),
        ('HOST_A', "localhost", True, 'The Web server and web socket host'),
        ('FILE_B', None, True, 'File containting regular expressions/grammar'),
        ('FILE_TYPE_B', "FLEX", True, 'File type (e.g FLEX, GRAMMAR, FST)'),
        ('WSPORT_B', "8002", True, 'The Web Socket server port'),
        ('WBPORT_B', "8082", True, 'The Web server port'),
        ('HOST_B', "localhost", True, 'The Web server and web socket host'),
        ('ALPHABET', None, False, 'File containing the alphabet'),
        ('CLASSIC', False, True, 'Use classic equivalence if sfadiff finds no counterexample'),
        ('DELAY', "50", True, 'Wait time for objects to load'),
    ],
    'comments': ['Sample comment 1', 'Sample comment 2']
}


class Module_A(CORE['SFADiff']):
    def __init__(self, configuration, channel, cross):

        super(Module_A, self).__init__(createalphabet(configuration['ALPHABET']), configuration['FILE_A'], configuration['FILE_TYPE_A'], channel,
                                     cross, configuration['CLASSIC'], False, True)

        self.browser_socket_handler = BrowserSocketHandler(int(configuration['WSPORT_A']), int(configuration['WBPORT_A']), True, configuration['DELAY'], configuration['HOST_A'])

    def _query(self, string):
        """
        Performs a membership query in the connected Browser
        Args:
            string: The string to be tested
        Returns:
            bool: A boolean value indicating that the target parser accepts or rejects the input
        """
        return self.browser_socket_handler.do_query(string)


class Module_B(CORE['SFADiff']):
    def __init__(self, configuration, channel, cross):

        super(Module_B, self).__init__(createalphabet(configuration['ALPHABET']), configuration['FILE_B'], configuration['FILE_TYPE_B'], channel,
                                     cross, configuration['CLASSIC'], False, True)

        self.browser_socket_handler = BrowserSocketHandler(int(configuration['WSPORT_B']), int(configuration['WBPORT_B']), True, configuration['DELAY'], configuration['HOST_B'])

    def _query(self, string):
        """
        Performs a membership query in the connected Browser
        Args:
            string: The string to be tested
        Returns:
            bool: A boolean value indicating that the target parser accepts or rejects the input
        """
        return self.browser_socket_handler.do_query(string)
