from lightbulb.core.base import CORE, createalphabet
from lightbulb.core.utils.http_handler import HTTPHandler
import imp
try:
    imp.find_module('MySQLdb')
    from lightbulb.core.utils.sql_handler import SQLHandler

    META = {
        'author': 'George Argyros, Ioannis Stais',
        'description': 'Find differences between a MySQL parser and a WAF filter',
        'type': 'SFADiff',
        'options': [
            ('FILE_A', None, True, 'File containting regular expressions/grammar'),
            ('FILE_TYPE_A', "FLEX", True, 'File type (e.g FLEX, GRAMMAR, FST)'),
            ('HOST_A', "127.0.0.1", True, 'The MySQL host'),
            ('PORT_A', "3306", True, 'The MySQL port'),
            ('USERNAME_A', None, True, 'The required username'),
            ('PASSWORD_A', None, True, 'The required password'),
            ('DATABASE_A', None, True, 'The MySQL database'),
            ('PREFIX_QUERY_A', 'SELECT id FROM a WHERE a = **;', True, 'The sql query to be concatenated'),
            ('FILE_B', None, True, 'File containting regular expressions/grammar'),
            ('FILE_TYPE_B', "FLEX", True, 'File type (e.g FLEX, GRAMMAR, FST)'),
            ('URL_B', "http://127.0.0.1", True, 'The target URL'),
            ('REQUEST_TYPE_B', "GET", True, 'The HTTP request type (GET/POST)'),
            ('PARAM_B', "input", True, 'The request parameter'),
            ('BLOCK_B', None, False, 'The response string that indicates that the WAF blocks the request'),
            ('BYPASS_B', None, False, 'The response string that indicates that the WAF allows the request'),
            ('PROXY_SCHEME', None, False, 'The proxy scheme (e.g. http, https'),
            ('PROXY_HOST', None, False, 'The proxy host'),
            ('PROXY_PORT', None, False, 'The proxy port'),
            ('PROXY_USERNAME', None, False, 'The proxy username'),
            ('PROXY_PASSWORD', None, False, 'The proxy password'),
            ('USER_AGENT', "Mozilla/5.0", True, 'The request user agent'),
            ('REFERER', "http://google.com", True, 'The request referrer'),
            ('ALPHABET', None, False, 'File containing the alphabet'),
            ('CLASSIC', False, True, 'Use classic equivalence if sfadiff finds no counterexample'),
        ],
        'comments': ['Sample comment 1', 'Sample comment 2']
    }


    class Module_A(CORE['SFADiff']):
        def __init__(self, configuration, channel, cross):

            super(Module_A, self).__init__(createalphabet(configuration['ALPHABET']), configuration['FILE_A'], configuration['FILE_TYPE_A'], channel,
                                         cross, configuration['CLASSIC'], True, True)

            self.sqlserver = SQLHandler(configuration['HOST_A'], int(configuration['PORT_A']),
                                        configuration['USERNAME_A'],
                                        configuration['PASSWORD_A'], configuration['DATABASE_A'],
                                        configuration['PREFIX_QUERY_A'], True)

        def _query(self, string):
            """
            Performs a membership query in the SQL database
            Args:
                string: The string to be tested
            Returns:
                bool: A boolean value indicating that the target parser accepts or rejects the input
            """
            return self.sqlserver.do_query(string)


    class Module_B(CORE['SFADiff']):
        def __init__(self, configuration, channel, cross):

            super(Module_B, self).__init__(createalphabet(configuration['ALPHABET']), configuration['FILE_B'], configuration['FILE_TYPE_B'], channel,
                                         cross, configuration['CLASSIC'], True, True)


            self.httphandler = HTTPHandler(configuration['URL_B'], configuration['REQUEST_TYPE_B'],
                                           configuration['PARAM_B'], configuration['BLOCK_B'],
                                           configuration['BYPASS_B'], configuration['PROXY_SCHEME'],
                                           configuration['PROXY_HOST'], configuration['PROXY_PORT'],
                                           configuration['PROXY_USERNAME'],
                                           configuration['PROXY_PASSWORD'], configuration['USER_AGENT'],
                                           configuration['REFERER'])

        def _query(self, string):
            """
            Performs a membership query in the targeted WAF
            Args:
                string: The string to be tested
            Returns:
                bool: A boolean value indicating that the target parser accepts or rejects the input
            """
            return self.httphandler.do_query(string)

except ImportError:
    pass