from lightbulb.core.base import CORE, options_as_dictionary, createalphabet
from lightbulb.core.utils.browserIframesocket_hander import BrowserIframeSocketHandler

META = {
        'author': 'George Argyros, Ioannis Stais',
        'description': 'Learns a Browser\'s anti XSS filter using a file containing regular expressions or grammar as input',
        'type':'GOFA',
        'options': [
            ('FILE', None, True, 'File containting regular expressions/grammar'),
            ('FILE_TYPE', "FLEX", True, 'File type (e.g FLEX, GRAMMAR, FST)'),
            ('WSPORT', "8000", True, 'The Web Socket server port'),
            ('WBPORT', "8080", True, 'The Web server port'),
            ('HOST', "localhost", True, 'The Web server and web socket host'),
            ('PRELOAD', True, True, 'Preload the input filter'),
            ('DELAY', "50", True, 'Wait time for objects to load'),
            ('ALPHABET', None, False, 'File containing the alphabet'),
            ('SAVE', None, False, 'File to save learned filter if no bypass is found'),
        ],
        'comments': ['Sample comment 1', 'Sample comment 2']
    }

class Module(CORE['GOFA']):


    def __init__(self, configuration):

        super(Module, self).__init__(createalphabet(configuration['ALPHABET']),  configuration['FILE'], configuration['FILE_TYPE'],  configuration['PRELOAD'], configuration['SAVE'])


        self.browser_socket_handler = BrowserIframeSocketHandler(int(configuration['WSPORT']), int(configuration['WBPORT']), False, configuration['DELAY'], configuration['HOST'])


    def _query(self, string):
        """
        Performs a membership query in the connected Browser
        Args:
            string: The string to be tested
        Returns:
            bool: A boolean value indicating that the target parser accepts or rejects the input
        """
        return self.browser_socket_handler.do_query(string)

