from lightbulb.core.base import CORE, createalphabet
from lightbulb.core.utils.http_handler import HTTPHandler

META = {
        'author': 'George Argyros, Ioannis Stais',
        'description': 'Learns a WAF filter using a file containing regular expressions or grammar as input',
        'type':'GOFA',
        'options': [
            ('FILE', None, True, 'File containting regular expressions/grammar'),
            ('FILE_TYPE', "FLEX", True, 'File type (e.g FLEX, GRAMMAR, FST)'),
            ('URL', "http://127.0.0.1", True, 'The target URL'),
            ('REQUEST_TYPE', "GET", True, 'The HTTP request type (GET/POST)'),
            ('PARAM', "input", True, 'The request parameter'),
            ('BLOCK', None, False, 'The response string that indicates that the WAF blocks the request'),
            ('BYPASS', None, False, 'The response string that indicates that the WAF allows the request'),
            ('PROXY_SCHEME', None, False, 'The proxy scheme (e.g. http, https'),
            ('PROXY_HOST', None, False, 'The proxy host'),
            ('PROXY_PORT', None, False, 'The proxy port'),
            ('PROXY_USERNAME', None, False, 'The proxy username'),
            ('PROXY_PASSWORD', None, False, 'The proxy password'),
            ('USER_AGENT', "Mozilla/5.0", True, 'The request user agent'),
            ('REFERER', "http://google.com", True, 'The request referrer'),
            ('PRELOAD', True, True, 'Preload the input filter'),
            ('ALPHABET', None, False, 'File containing the alphabet'),
            ('SAVE', None, False, 'File to save learned filter if no bypass is found'),

        ],
        'comments': ['Sample comment 1', 'Sample comment 2']
    }

class Module(CORE['GOFA']):


    def __init__(self, configuration):
        super(Module, self).__init__(createalphabet(configuration['ALPHABET']),  configuration['FILE'], configuration['FILE_TYPE'],  configuration['PRELOAD'], configuration['SAVE'])
        self.httphandler = HTTPHandler(configuration['URL'], configuration['REQUEST_TYPE'],
                            configuration['PARAM'], configuration['BLOCK'], configuration['BYPASS'], configuration['PROXY_SCHEME'],
                            configuration['PROXY_HOST'], configuration['PROXY_PORT'], configuration['PROXY_USERNAME'],
                            configuration['PROXY_PASSWORD'], configuration['USER_AGENT'], configuration['REFERER'])

    def _query(self, string):
        """
        Performs a membership query in the targeted WAF
        Args:
            string: The string to be tested
        Returns:
            bool: A boolean value indicating that the target parser accepts or rejects the input
        """
        return self.httphandler.do_query(string)

